@@ -1,4 +1,4 @@
-//          Copyright Naoki Shibata 2010 - 2019.
+//   Copyright Naoki Shibata and contributors 2010 - 2020.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -11,14 +11,6 @@
 #include <signal.h>
 #include <setjmp.h>
 
-#if defined(POWER64_UNDEF_USE_EXTERN_INLINES)
-// This is a workaround required to cross compile for PPC64 binaries
-#include <features.h>
-#ifdef __USE_EXTERN_INLINES
-#undef __USE_EXTERN_INLINES
-#endif
-#endif
-
 #include <math.h>
 
 #include "sleef.h"
@@ -251,7 +243,7 @@ static int omp_thread_count() {
 static void startAllThreads(const int nth) {
 #ifdef _OPENMP
   volatile int8_t *state = calloc(nth, 1);
-  int th;
+  int th=0;
 #pragma omp parallel for
   for(th=0;th<nth;th++) {
     state[th] = 1;
@@ -314,6 +306,19 @@ static void dispatch(SleefDFT *p, const int N, real *d, const real *s, const int
       *(element_t *)&row[x2].r[y2*2+0] = r;				\
     }} while(0)
 
+#if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
+// This is a workaround of a bug in gcc on s390
+static void transpose(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, const int log2n, const int log2m) {
+  for(int y=0;y<(1 << log2n);y++) {
+    for(int x=0;x<(1 << log2m);x++) {
+      real r0 = s[((y << log2m)+x)*2+0];
+      real r1 = s[((y << log2m)+x)*2+1];
+      d[((x << log2n)+y)*2+0] = r0;
+      d[((x << log2n)+y)*2+1] = r1;
+    }
+  }
+}
+#else
 static void transpose(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, const int log2n, const int log2m) {
   if (log2n < LOG2BS || log2m < LOG2BS) {
     for(int y=0;y<(1 << log2n);y++) {
@@ -364,8 +369,22 @@ static void transpose(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256)
     }
   }
 }
+#endif // #if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
 
 #ifdef _OPENMP
+#if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
+// This is a workaround of a bug in gcc on s390
+static void transposeMT(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, int log2n, int log2m) {
+  for(int y=0;y<(1 << log2n);y++) {
+    for(int x=0;x<(1 << log2m);x++) {
+      real r0 = s[((y << log2m)+x)*2+0];
+      real r1 = s[((y << log2m)+x)*2+1];
+      d[((x << log2n)+y)*2+0] = r0;
+      d[((x << log2n)+y)*2+1] = r1;
+    }
+  }
+}
+#else
 static void transposeMT(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, int log2n, int log2m) {
   if (log2n < LOG2BS || log2m < LOG2BS) {
     for(int y=0;y<(1 << log2n);y++) {
@@ -384,7 +403,7 @@ static void transposeMT(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(25
     typedef struct { real r[BS*2]; } row_t;
     typedef struct { real r0, r1; } element_t;
 #endif
-    int y;
+    int y=0;
 #pragma omp parallel for
     for(y=0;y<(1 << log2n);y+=BS) {
       for(int x=0;x<(1 << log2m);x+=BS) {
@@ -418,6 +437,7 @@ static void transposeMT(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(25
     }
   }
 }
+#endif // #if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
 #endif // #ifdef _OPENMP
 
 // Table generator
@@ -927,7 +947,7 @@ static void measureBut(SleefDFT *p) {
 	    if (p->tbl[N] == NULL || p->tbl[N][level] == NULL) continue;
 	    if (p->vecwidth > (1 << N)) continue;
 	    if ((config & CONFIG_MT) != 0) {
-	      int i1;
+	      int i1=0;
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
@@ -953,7 +973,7 @@ static void measureBut(SleefDFT *p) {
 	    if (p->vecwidth > 2 && p->log2len <= N+2) continue;
 	    if ((int)p->log2len - (int)level < p->log2vecwidth) continue;
 	    if ((config & CONFIG_MT) != 0) {
-	      int i1;
+	      int i1=0;
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
@@ -1388,7 +1408,7 @@ EXPORT void EXECUTE(SleefDFT *p, const real *s0, real *d0) {
 	(((p->mode & SLEEF_MODE_DEBUG) == 0 && p->tmMT < p->tmNoMT) ||
 	 ((p->mode & SLEEF_MODE_DEBUG) != 0 && (rand() & 1))))
       {
-	int y;
+	int y=0;
 #pragma omp parallel for
 	for(y=0;y<p->vlen;y++) {
 	  EXECUTE(p->instH, &s[p->hlen*2*y], &tBuf[p->hlen*2*y]);
