@@ -1,4 +1,4 @@
-//          Copyright Naoki Shibata 2010 - 2019.
+//   Copyright Naoki Shibata and contributors 2010 - 2020.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -6,14 +6,6 @@
 #ifndef __VECTORTYPE_H__
 #define __VECTORTYPE_H__
 
-#if defined(POWER64_UNDEF_USE_EXTERN_INLINES)
-// This is a workaround required to cross compile for PPC64 binaries
-#include <features.h>
-#ifdef __USE_EXTERN_INLINES
-#undef __USE_EXTERN_INLINES
-#endif
-#endif
-
 #include <math.h>
 #include "sleef.h"
 
@@ -49,6 +41,14 @@
 #include "helperpower_128.h"
 #endif
 
+#ifdef ENABLE_ZVECTOR
+#include "helpers390x_128.h"
+#endif
+
+#ifdef ENABLE_ZVECTOR2
+#include "helpers390x_128.h"
+#endif
+
 #ifdef ENABLE_VECEXT
 #include "helpervecext.h"
 #endif
