@@ -1,4 +1,4 @@
-//          Copyright Naoki Shibata 2010 - 2019.
+//   Copyright Naoki Shibata and contributors 2010 - 2020.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -7,10 +7,7 @@
 
 #ifdef ENABLE_DP
 #ifdef ENABLE_SVE
-typedef __sizeless_struct vdouble2 {
-  svfloat64_t x;
-  svfloat64_t y;
-} vdouble2;
+typedef svfloat64x2_t vdouble2;
 #else
 typedef struct {
   vdouble x, y;
@@ -94,6 +91,7 @@ vdouble xnextafter(vdouble, vdouble);
 vdouble xfrfrexp(vdouble);
 vint xexpfrexp(vdouble);
 vdouble xfmod(vdouble, vdouble);
+vdouble xremainder(vdouble, vdouble);
 vdouble2 xmodf(vdouble);
 
 vdouble xlgamma_u1(vdouble);
@@ -107,10 +105,7 @@ vdouble xerfc_u15(vdouble);
 
 #ifdef ENABLE_SP
 #ifdef ENABLE_SVE
-typedef __sizeless_struct vfloat2 {
-  svfloat32_t x;
-  svfloat32_t y;
-} vfloat2;
+typedef svfloat32x2_t vfloat2;
 #else
 typedef struct {
   vfloat x, y;
@@ -193,6 +188,7 @@ vfloat xnextafterf(vfloat, vfloat);
 vfloat xfrfrexpf(vfloat);
 vint2 xexpfrexpf(vfloat);
 vfloat xfmodf(vfloat, vfloat);
+vfloat xremainderf(vfloat, vfloat);
 vfloat2 xmodff(vfloat);
 
 vfloat xlgammaf_u1(vfloat);
