@@ -20,7 +20,7 @@
 
 #include "misc.h"
 
-extern const double rempitabdp[];
+extern const double Sleef_rempitabdp[];
 
 #ifdef DORENAME
 #include "rename.h"
@@ -53,15 +53,15 @@ static INLINE CONST double longBitsToDouble(int64_t i) {
 }
 
 static INLINE CONST double fabsk(double x) {
-  return longBitsToDouble(0x7fffffffffffffffLL & doubleToRawLongBits(x));
+  return longBitsToDouble(INT64_C(0x7fffffffffffffff) & doubleToRawLongBits(x));
 }
 
 static INLINE CONST double mulsign(double x, double y) {
-  return longBitsToDouble(doubleToRawLongBits(x) ^ (doubleToRawLongBits(y) & (1LL << 63)));
+  return longBitsToDouble(doubleToRawLongBits(x) ^ (doubleToRawLongBits(y) & (INT64_C(1) << 63)));
 }
 
 static INLINE CONST double copysignk(double x, double y) {
-  return longBitsToDouble((doubleToRawLongBits(x) & ~(1LL << 63)) ^ (doubleToRawLongBits(y) & (1LL << 63)));
+  return longBitsToDouble((doubleToRawLongBits(x) & ~(INT64_C(1) << 63)) ^ (doubleToRawLongBits(y) & (INT64_C(1) << 63)));
 }
 
 static INLINE CONST double sign(double d) { return mulsign(1, d); }
@@ -80,13 +80,13 @@ static INLINE CONST int xisnegzero(double x) { return doubleToRawLongBits(x) ==
 static INLINE CONST int xisnumber(double x) { return !xisinf(x) && !xisnan(x); }
 
 static INLINE CONST int xisint(double d) {
-  double x = d - (double)(1LL << 31) * (int)(d * (1.0 / (1LL << 31)));
-  return (x == (int)x) || (fabsk(d) >= (double)(1LL << 53));
+  double x = d - (double)(INT64_C(1) << 31) * (int)(d * (1.0 / (INT64_C(1) << 31)));
+  return (x == (int)x) || (fabsk(d) >= (double)(INT64_C(1) << 53));
 }
 
 static INLINE CONST int xisodd(double d) {
-  double x = d - (double)(1LL << 31) * (int)(d * (1.0 / (1LL << 31)));
-  return (1 & (int)x) != 0 && fabsk(d) < (double)(1LL << 53);
+  double x = d - (double)(INT64_C(1) << 31) * (int)(d * (1.0 / (INT64_C(1) << 31)));
+  return (1 & (int)x) != 0 && fabsk(d) < (double)(INT64_C(1) << 53);
 }
 
 static INLINE CONST double pow2i(int q) {
@@ -163,7 +163,7 @@ static int checkfp(double x) {
 #endif
 
 static INLINE CONST double upper(double d) {
-  return longBitsToDouble(doubleToRawLongBits(d) & 0xfffffffff8000000LL);
+  return longBitsToDouble(doubleToRawLongBits(d) & INT64_C(0xfffffffff8000000));
 }
 
 static INLINE CONST Sleef_double2 dd(double h, double l) {
@@ -655,7 +655,7 @@ static Sleef_double2 atan2k_u1(Sleef_double2 y, Sleef_double2 x) {
 }
 
 EXPORT CONST double xatan2_u1(double y, double x) {
-  if (fabsk(x) < 5.5626846462680083984e-309) { y *= (1ULL << 53); x *= (1ULL << 53); } // nexttoward((1.0 / DBL_MAX), 1)
+  if (fabsk(x) < 5.5626846462680083984e-309) { y *= (UINT64_C(1) << 53); x *= (UINT64_C(1) << 53); } // nexttoward((1.0 / DBL_MAX), 1)
   Sleef_double2 d = atan2k_u1(dd(fabsk(y), 0), dd(x, 0));
   double r = d.x + d.y;
 
@@ -747,16 +747,16 @@ typedef struct {
 } ddi_t;
 
 static INLINE CONST double orsign(double x, double y) {
-  return longBitsToDouble(doubleToRawLongBits(x) | (doubleToRawLongBits(y) & (1LL << 63)));
+  return longBitsToDouble(doubleToRawLongBits(x) | (doubleToRawLongBits(y) & (INT64_C(1) << 63)));
 }
 
 static CONST di_t rempisub(double x) {
   // This function is equivalent to :
   // di_t ret = { x - rint(4 * x) * 0.25, (int32_t)(rint(4 * x) - rint(x) * 4) };
   di_t ret;
-  double c = mulsign(1LL << 52, x);
-  double rint4x = fabsk(4*x) > 1LL << 52 ? (4*x) : orsign(mla(4, x, c) - c, x);
-  double rintx  = fabsk(  x) > 1LL << 52 ?   x   : orsign(x + c - c       , x);
+  double c = mulsign(INT64_C(1) << 52, x);
+  double rint4x = fabsk(4*x) > INT64_C(1) << 52 ? (4*x) : orsign(mla(4, x, c) - c, x);
+  double rintx  = fabsk(  x) > INT64_C(1) << 52 ?   x   : orsign(x + c - c       , x);
   ret.d = mla(-0.25, rint4x,      x);
   ret.i = mla(-4   , rintx , rint4x);
   return ret;
@@ -771,18 +771,18 @@ static CONST ddi_t rempi(double a) {
   a = ldexp3k(a, q);
   if (ex < 0) ex = 0;
   ex *= 4;
-  x = ddmul_d2_d_d(a, rempitabdp[ex]);
+  x = ddmul_d2_d_d(a, Sleef_rempitabdp[ex]);
   di = rempisub(x.x);
   q = di.i;
   x.x = di.d;
   x = ddnormalize_d2_d2(x);
-  y = ddmul_d2_d_d(a, rempitabdp[ex+1]);
+  y = ddmul_d2_d_d(a, Sleef_rempitabdp[ex+1]);
   x = ddadd2_d2_d2_d2(x, y);
   di = rempisub(x.x);
   q += di.i;
   x.x = di.d;
   x = ddnormalize_d2_d2(x);
-  y = ddmul_d2_d2_d(dd(rempitabdp[ex+2], rempitabdp[ex+3]), a);
+  y = ddmul_d2_d2_d(dd(Sleef_rempitabdp[ex+2], Sleef_rempitabdp[ex+3]), a);
   x = ddadd2_d2_d2_d2(x, y);
   x = ddnormalize_d2_d2(x);
   x = ddmul_d2_d2_d2(x, dd(3.141592653589793116*2, 1.2246467991473532072e-16*2));
@@ -905,8 +905,8 @@ EXPORT CONST double xcos(double d) {
     d = mla(ql, -PI_A2*0.5, d);
     d = mla(ql, -PI_B2*0.5, d);
   } else if (fabsk(d) < TRIGRANGEMAX) {
-    double dqh = trunck(d * (M_1_PI / (1LL << 23)) - 0.5 * (M_1_PI / (1LL << 23)));
-    ql = 2*rintk(d * M_1_PI - 0.5 - dqh * (double)(1LL << 23))+1;
+    double dqh = trunck(d * (M_1_PI / (INT64_C(1) << 23)) - 0.5 * (M_1_PI / (INT64_C(1) << 23)));
+    ql = 2*rintk(d * M_1_PI - 0.5 - dqh * (double)(INT64_C(1) << 23))+1;
     dqh *= 1 << 24;
 
     d = mla(dqh, -PI_A*0.5, d);
@@ -960,8 +960,8 @@ EXPORT CONST double xcos_u1(double d) {
     s = ddadd2_d2_d_d(d, ql * (-PI_A2*0.5));
     s = ddadd_d2_d2_d(s, ql * (-PI_B2*0.5));
   } else if (d < TRIGRANGEMAX) {
-    double dqh = trunck(d * (M_1_PI / (1LL << 23)) - 0.5 * (M_1_PI / (1LL << 23)));
-    ql = 2*rintk(d * M_1_PI - 0.5 - dqh * (double)(1LL << 23))+1;
+    double dqh = trunck(d * (M_1_PI / (INT64_C(1) << 23)) - 0.5 * (M_1_PI / (INT64_C(1) << 23)));
+    ql = 2*rintk(d * M_1_PI - 0.5 - dqh * (double)(INT64_C(1) << 23))+1;
     dqh *= 1 << 24;
 
     u = mla(dqh, -PI_A*0.5, d);
@@ -1440,7 +1440,7 @@ EXPORT CONST double xlog(double d) {
   int e;
 
   int o = d < DBL_MIN;
-  if (o) d *= (double)(1LL << 32) * (double)(1LL << 32);
+  if (o) d *= (double)(INT64_C(1) << 32) * (double)(INT64_C(1) << 32);
   
   e = ilogb2k(d * (1.0/0.75));
   m = ldexp3k(d, -e);
@@ -1533,7 +1533,7 @@ static INLINE CONST Sleef_double2 logk(double d) {
   int e;
 
   int o = d < DBL_MIN;
-  if (o) d *= (double)(1LL << 32) * (double)(1LL << 32);
+  if (o) d *= (double)(INT64_C(1) << 32) * (double)(INT64_C(1) << 32);
   
   e = ilogb2k(d * (1.0/0.75));
   m = ldexp3k(d, -e);
@@ -1572,7 +1572,7 @@ EXPORT CONST double xlog_u1(double d) {
   int e;
 
   int o = d < DBL_MIN;
-  if (o) d *= (double)(1LL << 32) * (double)(1LL << 32);
+  if (o) d *= (double)(INT64_C(1) << 32) * (double)(INT64_C(1) << 32);
       
   e = ilogb2k(d * (1.0/0.75));
   m = ldexp3k(d, -e);
@@ -2041,7 +2041,7 @@ EXPORT CONST double xlog10(double d) {
   int e;
 
   int o = d < DBL_MIN;
-  if (o) d *= (double)(1LL << 32) * (double)(1LL << 32);
+  if (o) d *= (double)(INT64_C(1) << 32) * (double)(INT64_C(1) << 32);
       
   e = ilogb2k(d * (1.0/0.75));
   m = ldexp3k(d, -e);
@@ -2080,7 +2080,7 @@ EXPORT CONST double xlog2(double d) {
   int e;
 
   int o = d < DBL_MIN;
-  if (o) d *= (double)(1LL << 32) * (double)(1LL << 32);
+  if (o) d *= (double)(INT64_C(1) << 32) * (double)(INT64_C(1) << 32);
       
   e = ilogb2k(d * (1.0/0.75));
   m = ldexp3k(d, -e);
@@ -2117,7 +2117,7 @@ EXPORT CONST double xlog2_u35(double d) {
   int e;
 
   int o = d < DBL_MIN;
-  if (o) d *= (double)(1LL << 32) * (double)(1LL << 32);
+  if (o) d *= (double)(INT64_C(1) << 32) * (double)(INT64_C(1) << 32);
       
   e = ilogb2k(d * (1.0/0.75));
   m = ldexp3k(d, -e);
@@ -2153,7 +2153,7 @@ EXPORT CONST double xlog1p(double d) {
   double dp1 = d + 1;
   
   int o = dp1 < DBL_MIN;
-  if (o) dp1 *= (double)(1LL << 32) * (double)(1LL << 32);
+  if (o) dp1 *= (double)(INT64_C(1) << 32) * (double)(INT64_C(1) << 32);
       
   e = ilogb2k(dp1 * (1.0/0.75));
 
@@ -2194,14 +2194,14 @@ EXPORT CONST double xlog1p(double d) {
 EXPORT CONST double xfma(double x, double y, double z) {
   double h2 = x * y + z, q = 1;
   if (fabsk(h2) < 1e-300) {
-    const double c0 = 1ULL << 54, c1 = c0 * c0, c2 = c1 * c1;
+    const double c0 = UINT64_C(1) << 54, c1 = c0 * c0, c2 = c1 * c1;
     x *= c1;
     y *= c1;
     z *= c2;
     q = 1.0 / c2;
   }
   if (fabsk(h2) > 1e+299) {
-    const double c0 = 1ULL << 54, c1 = c0 * c0, c2 = c1 * c1;
+    const double c0 = UINT64_C(1) << 54, c1 = c0 * c0, c2 = c1 * c1;
     x *= 1.0 / c1;
     y *= 1.0 / c1;
     z *= 1. / c2;
@@ -2268,38 +2268,38 @@ EXPORT CONST double xfdim(double x, double y) {
 }
 
 EXPORT CONST double xtrunc(double x) {
-  double fr = x - (double)(1LL << 31) * (int32_t)(x * (1.0 / (1LL << 31)));
+  double fr = x - (double)(INT64_C(1) << 31) * (int32_t)(x * (1.0 / (INT64_C(1) << 31)));
   fr = fr - (int32_t)fr;
-  return (xisinf(x) || fabsk(x) >= (double)(1LL << 52)) ? x : copysignk(x - fr, x);
+  return (xisinf(x) || fabsk(x) >= (double)(INT64_C(1) << 52)) ? x : copysignk(x - fr, x);
 }
 
 EXPORT CONST double xfloor(double x) {
-  double fr = x - (double)(1LL << 31) * (int32_t)(x * (1.0 / (1LL << 31)));
+  double fr = x - (double)(INT64_C(1) << 31) * (int32_t)(x * (1.0 / (INT64_C(1) << 31)));
   fr = fr - (int32_t)fr;
   fr = fr < 0 ? fr+1.0 : fr;
-  return (xisinf(x) || fabsk(x) >= (double)(1LL << 52)) ? x : copysignk(x - fr, x);
+  return (xisinf(x) || fabsk(x) >= (double)(INT64_C(1) << 52)) ? x : copysignk(x - fr, x);
 }
 
 EXPORT CONST double xceil(double x) {
-  double fr = x - (double)(1LL << 31) * (int32_t)(x * (1.0 / (1LL << 31)));
+  double fr = x - (double)(INT64_C(1) << 31) * (int32_t)(x * (1.0 / (INT64_C(1) << 31)));
   fr = fr - (int32_t)fr;
   fr = fr <= 0 ? fr : fr-1.0;
-  return (xisinf(x) || fabsk(x) >= (double)(1LL << 52)) ? x : copysignk(x - fr, x);
+  return (xisinf(x) || fabsk(x) >= (double)(INT64_C(1) << 52)) ? x : copysignk(x - fr, x);
 }
 
 EXPORT CONST double xround(double d) {
   double x = d + 0.5;
-  double fr = x - (double)(1LL << 31) * (int32_t)(x * (1.0 / (1LL << 31)));
+  double fr = x - (double)(INT64_C(1) << 31) * (int32_t)(x * (1.0 / (INT64_C(1) << 31)));
   fr = fr - (int32_t)fr;
   if (fr == 0 && x <= 0) x--;
   fr = fr < 0 ? fr+1.0 : fr;
   x = d == 0.49999999999999994449 ? 0 : x;  // nextafter(0.5, 0)
-  return (xisinf(d) || fabsk(d) >= (double)(1LL << 52)) ? d : copysignk(x - fr, d);
+  return (xisinf(d) || fabsk(d) >= (double)(INT64_C(1) << 52)) ? d : copysignk(x - fr, d);
 }
 
 EXPORT CONST double xrint(double d) {
-  double c = mulsign(1LL << 52, d);
-  return fabsk(d) > 1LL << 52 ? d : orsign(d + c - c, d);
+  double c = mulsign(INT64_C(1) << 52, d);
+  return fabsk(d) > INT64_C(1) << 52 ? d : orsign(d + c - c, d);
 }
 
 EXPORT CONST double xhypot_u05(double x, double y) {
@@ -2308,7 +2308,7 @@ EXPORT CONST double xhypot_u05(double x, double y) {
   double min = fmink(x, y), n = min;
   double max = fmaxk(x, y), d = max;
 
-  if (max < DBL_MIN) { n *= 1ULL << 54; d *= 1ULL << 54; }
+  if (max < DBL_MIN) { n *= UINT64_C(1) << 54; d *= UINT64_C(1) << 54; }
   Sleef_double2 t = dddiv_d2_d2_d2(dd(n, 0), dd(d, 0));
   t = ddmul_d2_d2_d(ddsqrt_d2_d2(ddadd2_d2_d2_d(ddsqu_d2_d2(t), 1)), max);
   double ret = t.x + t.y;
@@ -2342,11 +2342,11 @@ EXPORT CONST double xnextafter(double x, double y) {
   x = x == 0 ? mulsign(0, y) : x;
   cx.f = x;
   int c = (cx.i < 0) == (y < x);
-  if (c) cx.i = -(cx.i ^ (1ULL << 63));
+  if (c) cx.i = -(cx.i ^ (UINT64_C(1) << 63));
 
   if (x != y) cx.i--;
 
-  if (c) cx.i = -(cx.i ^ (1ULL << 63));
+  if (c) cx.i = -(cx.i ^ (UINT64_C(1) << 63));
 
   if (cx.f == 0 && x != 0) cx.f = mulsign(0, x);
   if (x == 0 && y == 0) cx.f = y;
@@ -2361,11 +2361,11 @@ EXPORT CONST double xfrfrexp(double x) {
     uint64_t u;
   } cx;
 
-  if (fabsk(x) < DBL_MIN) x *= (1ULL << 63);
+  if (fabsk(x) < DBL_MIN) x *= (UINT64_C(1) << 63);
   
   cx.f = x;
-  cx.u &= ~0x7ff0000000000000ULL;
-  cx.u |=  0x3fe0000000000000ULL;
+  cx.u &= ~UINT64_C(0x7ff0000000000000);
+  cx.u |=  UINT64_C(0x3fe0000000000000);
 
   if (xisinf(x)) cx.f = mulsign(SLEEF_INFINITY, x);
   if (x == 0) cx.f = x;
@@ -2381,7 +2381,7 @@ EXPORT CONST int xexpfrexp(double x) {
 
   int ret = 0;
   
-  if (fabsk(x) < DBL_MIN) { x *= (1ULL << 63); ret = -63; }
+  if (fabsk(x) < DBL_MIN) { x *= (UINT64_C(1) << 63); ret = -63; }
   
   cx.f = x;
   ret += (int32_t)(((cx.u >> 52) & 0x7ff)) - 0x3fe;
@@ -2396,17 +2396,17 @@ static INLINE CONST double toward0(double d) {
 }
 
 static INLINE CONST double removelsb(double d) {
-  return longBitsToDouble(doubleToRawLongBits(d) & 0xfffffffffffffffeLL);
+  return longBitsToDouble(doubleToRawLongBits(d) & INT64_C(0xfffffffffffffffe));
 }
 
 static INLINE CONST double ptrunc(double x) {
-  double fr = mla(-(double)(1LL << 31), (int32_t)(x * (1.0 / (1LL << 31))), x);
-  return fabsk(x) >= (double)(1LL << 52) ? x : (x - (fr - (int32_t)fr));
+  double fr = mla(-(double)(INT64_C(1) << 31), (int32_t)(x * (1.0 / (INT64_C(1) << 31))), x);
+  return fabsk(x) >= (double)(INT64_C(1) << 52) ? x : (x - (fr - (int32_t)fr));
 }
 
 EXPORT CONST double xfmod(double x, double y) {
   double n = fabsk(x), d = fabsk(y), s = 1, q;
-  if (d < DBL_MIN) { n *= 1ULL << 54; d *= 1ULL << 54; s = 1.0 / (1ULL << 54); }
+  if (d < DBL_MIN) { n *= UINT64_C(1) << 54; d *= UINT64_C(1) << 54; s = 1.0 / (UINT64_C(1) << 54); }
   Sleef_double2 r = dd(n, 0);
   double rd = toward0(1.0 / d);
   
@@ -2429,13 +2429,13 @@ EXPORT CONST double xfmod(double x, double y) {
 }
 
 static INLINE CONST double rintk2(double d) {
-  double c = mulsign(1LL << 52, d);
-  return fabsk(d) > 1LL << 52 ? d : orsign(d + c - c, d);
+  double c = mulsign(INT64_C(1) << 52, d);
+  return fabsk(d) > INT64_C(1) << 52 ? d : orsign(d + c - c, d);
 }
 
 EXPORT CONST double xremainder(double x, double y) {
   double n = fabsk(x), d = fabsk(y), s = 1, q;
-  if (d < DBL_MIN*2) { n *= 1ULL << 54; d *= 1ULL << 54; s = 1.0 / (1ULL << 54); }
+  if (d < DBL_MIN*2) { n *= UINT64_C(1) << 54; d *= UINT64_C(1) << 54; s = 1.0 / (UINT64_C(1) << 54); }
   double rd = 1.0 / d;
   Sleef_double2 r = dd(n, 0);
   int qisodd = 0;
@@ -2459,9 +2459,9 @@ EXPORT CONST double xremainder(double x, double y) {
 }
 
 EXPORT CONST Sleef_double2 xmodf(double x) {
-  double fr = x - (double)(1LL << 31) * (int32_t)(x * (1.0 / (1LL << 31)));
+  double fr = x - (double)(INT64_C(1) << 31) * (int32_t)(x * (1.0 / (INT64_C(1) << 31)));
   fr = fr - (int32_t)fr;
-  fr = fabsk(x) >= (double)(1LL << 52) ? 0 : fr;
+  fr = fabsk(x) >= (double)(INT64_C(1) << 52) ? 0 : fr;
   Sleef_double2 ret = { copysignk(fr, x), copysignk(x - fr, x) };
   return ret;
 }
@@ -2533,9 +2533,9 @@ static CONST dd2 gammak(double a) {
     (oref ? ddadd2_d2_d2_d2(dd(1.1447298858494001639, 1.026595116270782638e-17), ddneg_d2_d2(clc)) : clc); // log(M_PI)
   clln = otiny ? dd(1, 0) : (oref ? clln : clld);
 
-  if (oref) x = ddmul_d2_d2_d2(clld, sinpik(a - (double)(1LL << 28) * (int32_t)(a * (1.0 / (1LL << 28)))));
+  if (oref) x = ddmul_d2_d2_d2(clld, sinpik(a - (double)(INT64_C(1) << 28) * (int32_t)(a * (1.0 / (INT64_C(1) << 28)))));
 
-  clld = otiny ? dd(a*((1LL << 60)*(double)(1LL << 60)), 0) : (oref ? x : y);
+  clld = otiny ? dd(a*((INT64_C(1) << 60)*(double)(INT64_C(1) << 60)), 0) : (oref ? x : y);
 
   dd2 ret = { clc, dddiv_d2_d2_d2(clln, clld) };
 
