@@ -1,4 +1,4 @@
 #!/bin/bash
 set -ev
 brew update
-brew install openssl
+brew install openssl fftw ninja
