@@ -2,8 +2,8 @@
 set -ev
 export QEMU_CPU=POWER8
 cd /build/build-cross
-make -j 2 all
+ninja all
 export OMP_WAIT_POLICY=passive
 export CTEST_OUTPUT_ON_FAILURE=TRUE
-ctest -j 2
-make install
+ctest -j `nproc`
+ninja install
