@@ -1,7 +1,8 @@
 #!/bin/bash
 set -ev
 cd sleef.build
-make -j 2 all
+ninja
+export OMP_WAIT_POLICY=passive
 export CTEST_OUTPUT_ON_FAILURE=TRUE
-ctest --verbose -j 2
-make install
+ctest -j `nproc`
+ninja install
